//
// fbuf layer implementation
//
#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include "fbuf.h"



// initialize a filebuffer structure
fbuf::fbuf(int nsnode, long *snoffset, int *snodsize, int fid)
{
  for (int k = 0; k < NBUFS; k++)
    {
      buffer[k] = NULL;
      offset[k] = 0L;
      size[k]   = 0;
      dirty[k]  = 0;
      locks[k]  = 0;
    }

  rip      = 0;
  fd       = fid;
  numsnode = nsnode;
  snoffs   = new long[numsnode];
  snsize   = new int[numsnode];
  
  for (k = 0; k < numsnode; k++)
    {
      snoffs[k] = snoffset[k];
      snsize[k] = snodsize[k];
    }
}



char *fbuf::fbget(int snode, long offset)
{
  int ndx = this->fsearch(snoffs[snode], snsize[snode]);

  if (ndx == -1)
    {
      ndx = this->falloc(snode);
    }

  if (offset < 0L)
    return this->foffset(ndx, snoffs[snode]);
  else
    return this->foffset(ndx, offset);
}





void fbuf::fbaccess(int numaccess, int *list)
{
  for (int k = 0; k < numaccess; k++)
    if (this->fsearch(snoffs[list[k]], snsize[list[k]]) < 0)
      if (this->falloc_check(list[k]) < 0)
	return;
}




char *fbuf::fbread(int sno, long offset, int size)
{
  int ndx = this->fsearch(offset, size);
  
  if (ndx == -1)
    {
      return this->fbget(sno, offset);
    }

  return this->foffset(ndx, offset);
}



void fbuf::fbwrite(char *buf, long offset, int size)
{
  int ndx = this->fsearch(offset, size);
  
  if (ndx == -1)
    {
      cerr << "Error - cannot find write buffer\n";
      exit(1);
    }

  dirty[ndx] = 1;

#if 0  // should be locked already!
  char *dest = this->foffset(ndx, offset);
  for (int k = 0; k < size; k++)
    dest[k] = buf[k];
#endif
}



void fbuf::flock(int snode)
{
  int ndx = this->fsearch(snoffs[snode], snsize[snode]);
  if (ndx == -1)
    {
      cout << "Unlock failed, buffer not in memory (sn = " << snode << ") ";
      cout << "Offset = " << snoffs[snode] << " Size = " << snsize[snode] << endl;
      this->status();
      exit(1);
    }
  locks[ndx]++;
}




void fbuf::funlock(int snode)
{
  int ndx = this->fsearch(snoffs[snode], snsize[snode]);
  if (ndx == -1)
    {
      cout << "Unlock failed, buffer not in memory (sn = " << snode << ")." << endl;
      this->status();
      exit(1);
    }
  if (locks[ndx] < 1)
    {
      cerr << "Unlock failed, no lock on the buffer (sn = " << snode << ")." << endl;
      this->status();
      exit(1);
    }
  locks[ndx]--;
}





int fbuf::fsearch(long ofs, int sz)
{
  for (int k = 0; k < NBUFS; k++)
    if (buffer[k])
      if ((ofs>=offset[k]) && ((ofs+sz)<=(offset[k]+size[k]))) return k;

  return -1;
}




int fbuf::falloc(int sno, int mode)
{
  int ndx = -1, j, sz;

  // find a suitable spot for getting new data
  for (j = 0; j < NBUFS; j++, rip = (rip+1) % NBUFS)
    if (locks[rip] == 0) 
      {
	ndx = rip;
	rip++;
	break;
      }

  if ((ndx == -1) && (mode == 1)) return ndx;
  else if (ndx == -1)
    {
      cerr << "Falloc failed, check lock manager...\n";
      this->status();
      exit(1);
    }

  // decide how much to read in 
  for (sz = 0, j = sno; (j < numsnode) && (sz < MINBUF); j++)
    sz += snsize[j];

  // cleanup any existing data
  if (buffer[ndx])
    {
      if (dirty[ndx])
	{
	  lseek(fd, offset[ndx], SEEK_SET);
	  write(fd, buffer[ndx], size[ndx]);
	}
      if (sz > size[ndx])
	{
	  delete[] buffer[ndx];
	  buffer[ndx] = new char[sz];
	}
    }
  else
    {
      buffer[ndx] = new char[sz];
    }

  // read afresh
  size[ndx]   = sz;
  offset[ndx] = snoffs[sno];
  dirty[ndx]  = 0;
  locks[ndx]  = 0;

  lseek(fd, offset[ndx], SEEK_SET);
  read(fd, buffer[ndx], sz);

  return ndx;
}




int fbuf::falloc_check(int sno)
{
  return this->falloc(sno, 1);
}




char *fbuf::foffset(int ndx, long ofs)
{
  return buffer[ndx] + (ofs - offset[ndx]);
}




void fbuf::status(void)
{
  cout << "BUF\tOffset\tSize\tState\tLock\t(RIP = " << rip << ")\n";
  for (int k = 0; k < NBUFS; k++)
    {
      cout << k << "\t" << offset[k] << "\t" << size[k] << "\t" << dirty[k];
      cout << "\t" << locks[k] << endl;
    }
}
