//
// sp.h
//
// Implementation of Message Passing API for IBM SP Systems.
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
//
# if !defined(__SP_H__)
#define __SP_H__

#include <iostream.h>
#include <mpproto.h>
#include "mpapi.h"

#include "vector.h"
#include "buffer.h"

#include "spmpl.h"

class ibmsp : public mpapi {
public:
  ibmsp( void )
    {
      mpc_environ(&np, &me);
    };

  void init(void)
    {
      mpc_environ(&np, &me);
    };

  int asend(int to, int tag, int size, void *buf)
    {
      return mpl_asend(to, tag, size, buf);
    }

  int arecv(int &tag, int &src, int size, void *buf)
    {
      return mpl_arecv(&tag, &src, size, buf);
    }

  int mwait(int id)
    {
      return mpl_mwait(id);
    }

  void bsend(int to, int tag, int size, void *buf)
    {
      //cout << "\tsend to:" << to << " tag: " << tag << " siz: " << size << "\n";
      //cout.flush();
      mpl_bsend(to,tag,size,buf);
    }

  void brecv(int &tag, int &src, int &size, void *buf)
    {
      //cout << "\trecv fm:" << src << " tag: " << tag << " siz: " << size << "\n";
      //cout.flush();
      mpl_brecv(&tag, &src, &size, buf);
    }

  void broadcast(int root, int tag, int size, void *buf)
    {
      if (me == root)
	{
	  for (int k = 0; k < np; k++)
	    if (k != me)
	      this->bsend(k, tag, size, buf);
	}
      else
	{
	  this->brecv(tag, root, size, buf);
	}
    }

  void multicast(Vector *set, int tag, int size, void *buffer)
    {
      //cout << "\tMultiCast(" << me << ", " << tag << ") ";
      // set->print("Target");
      for (int k = 0; k < set->getSize(); k++)
	if (set->getElem(k) != me)
	  this->bsend(set->getElem(k), tag, size, buffer);
    }

  void bufCollect(Buffer &mybuf, int tagbase)
    {
      Buffer tbuf(mybuf);
      for (int k = 0; k < np; k++)
	if (me == k)
	  {
	    int size = tbuf.numUsed();
	    this->broadcast(me, tagbase+me, sizeof(int), &size);
	    this->broadcast(me, tagbase+me*2, size*sizeof(int), tbuf.getData());
	  }
	else
	  {
	    int tag = tagbase + k;
	    int src = k;
	    int siz = sizeof(int);
	    int cnt;
	    this->brecv(tag, src, siz, &cnt);
	    int *buf = new int[cnt];
	    tag = tagbase + 2*k;
	    siz = cnt * sizeof(int);
	    this->brecv(tag, src, siz, buf);

	    mybuf.merge(cnt, buf);
	    delete[] buf;
	  }
    }
};

#endif
