//
// vector.h
//
// Vector operations library
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
#if !defined(__VECTOR_H__)
#define __VECTOR_H__

#include <iostream.h>
#include <assert.h>

class Vector {
  int  size;
  int *data;

public:
  Vector( void )
  {
    size = 0;
    data = NULL;
  };

  Vector(int cnt, int val = 0)
  {
    if (cnt>0)
      {
	size = cnt;
	data = new int[size];
	assert( data );
	for (int i = 0; i < size; i++)
	  data[i] = val;
      }
    else
      {
	size = 0;
	data = NULL;
      }
  };

  Vector(int cnt, int *ndata)
    {
      if (cnt > 0)
	{
	  size = cnt;
	  data = new int[cnt];
	  assert( data );
	  for (int k = 0; k < cnt; k++)
	    data[k] = ndata[k];
	}
      else
	{
	  size = 0;
	  data = NULL;
	}
    }

  void purge( void )
    {
    if (size>0) delete[] data;

    size = 0;
    data = NULL;
    };

  ~Vector( void )
  {
    this->purge();
  };

  int checkIndex(int ndx)
  {
    if ((ndx<0)||(ndx>size))
      {
	cerr << "\tError : index out of range, ndx = " << ndx << ".\n";
	return 1;
      }
    
    return 0;
  };

  int getSize(void)
  {
    return size;
  };

  int *getData(void)
    {
      return data;
    }


  void print( char *msg )
    {
      cout << msg << ":<" << size << "> [";
      for (int k = 0; k < size; k++)
	cout << data[k] << " ";
      cout << "]\n";
      cout.flush();
    }

  void setElem(int ndx, int elem)
  {
    if (checkIndex(ndx)) return;
    data[ndx] = elem;
  };

  int getElem(int ndx)
  {
    if (checkIndex(ndx)) return 0;
    else return data[ndx];
  };


  void delElem(int elem)
  {
    for (int k = 0; k < size; k++)
      if (data[k] == elem)
	{
	  data[k] = data[--size];
	  k--;
	}
  }


  // expand vector to cover more data 
  void expand(int extra)
  {
    assert( extra > 0 );

    int  nsiz = extra + size;
    int *nbuf = new int[nsiz];
    assert( nbuf != 0 );

    if (size>0)
      {
	for (int k = 0; k < size; k++)
	  nbuf[k] = data[k];
	delete[] data;
      }

    data = nbuf;
    size = nsiz;
  };

  void merge(int cnt, int valid, int *ndata)
  {
    if (cnt <= 0) return;
    int *nbuf = new int[cnt+size];
    int  i = 0, j = 0;

    for (int k = 0; (i < valid) && (j < cnt); k++)
      if (data[i] < ndata[j]) nbuf[k] = data[i++];
      else nbuf[k] = ndata[j++];
    while (i < valid)
      nbuf[k++] = data[i++];
    while (j < cnt)
      nbuf[k++] = ndata[j++];
    
    if (data) delete[] data;
    data  = nbuf;
    size += cnt;
  }

  void sort( int cnt )
  {
    for (int i = 0; i < cnt; i++)
      for (int j = i+1; j < cnt; j++)
	if (data[j]<data[i])
	  {
	    int tmp = data[j];
	    data[j] = data[i];
	    data[i] = tmp;
	  }
  }
};


#endif
