/*
 * block.c
 *
 * $Source: /fs/nene/uysal/npFact/RCS/block.c,v $
 *
 * $Header: /fs/nene/uysal/npFact/RCS/block.c,v 1.1 1995/10/16 03:42:25 uysal Exp $
 *
 * Module to implement the management of blocks
 *
 * Author : Mustafa Uysal (uysal@cs.umd.edu)
 *
 */
#include <stdio.h>
#include <stdlib.h>

#include "block.h"
#include "queue.h"
#include "snode.h"


/*--------------- Module Implementation ---------------*/



/*
 * Routine to determine the size of a block (in words)
 */
int BlockSize( blk )
     block_t *blk;
{
  int tmp;
  int res;

  if (blk->xcoor == blk->ycoor)
    {
      tmp = blk->rcol - blk->lcol;
      res = ((tmp * (tmp + 1)) / 2);
    }
  else
    {
      res = ((blk->lrow - blk->urow) * (blk->rcol - blk->lcol));
    }

#if defined(DEBUG)
  if ((res<=0)||(res>20000))
    {
      message(("Suspicious block size for Block[%d %d] : %d -> Params[%d %d %d %d]",
	   blk->xcoor, blk->ycoor, res, blk->lcol, blk->rcol, blk->urow, blk->lrow));
#if 0
      tmp = 0;
      *((int*)tmp) = 1;  /* SEGV */
#endif
    }
#endif

  return res;
}



/*
 * Routine to determine the size of a block (in bytes)
 */
int BlockSizeByte( blk )
     block_t *blk;
{
  int nnz;

  nnz = BlockSize(blk);
  return (nnz * sizeof(nonzero_t));
}




/*
 * Print the attributes of a block structure
 */
void BlockPrint(blk, msg)
     block_t *blk;
     char    *msg;
{
  int size;

  size = BlockSizeByte(blk);
  printf("\n%s: (%d,%d):-[[%d %d] to [%d %d] %d] %.12lE", msg, blk->xcoor,
	 blk->ycoor, blk->urow, blk->lcol, blk->lrow, blk->rcol, size, blk->value[0]);
}









