/*
 * block.h
 *
 * $Source: /fs/nene/uysal/npFact/RCS/block.h,v $
 *
 * $Header: /fs/nene/uysal/npFact/RCS/block.h,v 1.1 1995/10/16 03:42:25 uysal Exp $
 *
 * Export module to define the interface for blocks
 *
 * Author : Mustafa Uysal (uysal@cs.umd.edu)
 *
 */
#ifndef __BLOCK_H__
#define __BLOCK_H__

#include "world.h"
#include "datatype.h"



/*--------------- Type Definitions ---------------*/

typedef struct block {
  /* attribute data for the block */
  int  xcoor;       /* x-coordinate of the block in 2D grid */
  int  ycoor;       /* y-coordinate of the block in 2D grid */
  int  lcol;        /* index of col bounding from left */
  int  rcol;        /* index of col bounding from right */
  int  urow;        /* index of row bounding from up */
  int  lrow;        /* index of row bounding from down */

  /* nonzeros in the block */
  nonzero_t *value; /* Numerical values of the block's nonzeros */
} block_t;



/*--------------- Exported Functions ---------------*/

/* Routine to be called when a block space is free to be used again */
void BlockFinished ARGS((snode_t *refptr));

/* Function to compute the size of a block in words */
int BlockSize ARGS((block_t *blk));

/* Same as blocksize, but the size is returned in bytes */
int BlockSizeByte ARGS((block_t *blk));

/* Utility function to print the attributes of a block */
void BlockPrint ARGS((block_t *blk, char *msg));

#endif  __BLOCK_H__

