/*
 * snode.c
 *
 * $Source: /fs/nene/uysal/npFact/RCS/snode.c,v $
 *
 * $Header: /fs/nene/uysal/npFact/RCS/snode.c,v 1.2 1995/12/27 02:00:09 uysal Exp $
 *
 * Resource management and dependency control modules for
 * supernodes
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include <malloc.h>

#include "snode.h"
#include "disk.h"
#include "queue.h"

#include "debug.h"


/*--------------- Static Variables ---------------*/

static int snCount;
static int *Offset;
static int *Size;

static snode_t **snStore;


void SnodeFreeForced ARGS((snode_t *ptr));
void SnodeCompact ARGS((int id, int size));


/*--------------- Implementation ---------------*/


/*
 * Initialization of supernode structures altogether
 */
void SnodeInit(cnt, offset, size)
    int cnt;
    int *offset, *size;
{
  int i;

  snCount = cnt;
  Offset = offset;
  Size = size;

  snStore = (snode_t**) Malloc(cnt*sizeof(snode_t**));
  for (i=0; i<cnt; i++)
    {
      snStore[i] = (snode_t*) NULL;
      Size[i] = -Size[i];
    }
}




/*
 * Extracts the list of updaters for a particular supernode
 * from the supernode data just read from the disk. 
 */
int *SnodeUpdaters(sptr, cnt)
    snode_t *sptr;
    int *cnt;
{
  int *ptr = (int*)sptr->buffer;

  *cnt = *ptr;
  return(ptr+1);
}




/*
 * Allocate memory for a supernode data + metadata 
 */
snode_t *SnodeAlloc(id, offset, size)
    int id, offset, size;
{
  caddr_t memptr;
  snode_t *sptr;

  /* Allocate buffer space if available */
  memptr = Malloc(sizeof(snode_t) + size);

  snStore[id] = sptr = (snode_t*) memptr;
  sptr->buffer = memptr + sizeof(snode_t);

  sptr->sid = id;
  sptr->offset = offset;
  sptr->size = size;

  sptr->blkcnt = 0;
  sptr->blkPtr = NULL;

  return sptr;
}




/* Routine to reclaim memory allocated by a supernode */
void SnodeFree( ptr )
    snode_t *ptr;
{
  if (!ptr) return;

  exit_if(ptr->sid<0||ptr->sid>=snCount, ("\nInvalid id:%d", ptr->sid));
  
  /* take the supernode off-line */
  snStore[ptr->sid] = (snode_t*) NULL;

  /* reclaim main buffer */  
  Free(ptr);
}


/* Routine to service supernode requests from memory if possible */
snode_t *SnodeRequest(id)
    int id;
{
  exit_if(id<0||id>snCount, ("Invalid ID:%d", id));
  return snStore[id];
}



/* Routine to query the disk offset where a supernode is located */
int SnodeOffset( id )
    int id;
{
  return Offset[id];
}



/* Routine to query the size of a supernode on disk */
int SnodeSize(id)
    int id;
{
  return Size[id];
}



/* Routine to adjust the offset & size of a supernode */
void  SnodeLocatorAdjust(id, diff )
    int id, diff;
{
  assert(Size[id]<0);

  Offset[id] += diff;
  Size[id] = (-Size[id]) - diff;
  exit_if(Size[id]<0, ("Locator Error size:(%d) for %d, diff=%d", Size[id], id, diff));
}





void SnodeExit()
{
  Free(snStore);
}



void SnodeStructPrint( sptr )
    snode_t *sptr;
{
  block_t *bptr;
  int j;
  
  for (j=0; j<sptr->blkcnt; j++)
    {
      bptr = &sptr->blkPtr[j];
      message(("\n\tBlk[%d of %d] is [%d %d @ %d %d %d %d] sz:%d", j, sptr->sid,
	       bptr->ycoor, bptr->xcoor, bptr->lcol, bptr->rcol,
	       bptr->urow, bptr->lrow, BlockSizeByte(bptr)));
    }
}
