/*
 * world.h
 *
 * $Source: /fs/nene/uysal/npFact/RCS/world.h,v $
 *
 * $Header: /fs/nene/uysal/npFact/RCS/world.h,v 1.1 1995/10/16 03:42:25 uysal Exp $
 *
 * This file contains definitions of an environment that each source file 
 * should have. It basically defines the "world" that a source file lives in.
 *
 * Author : Mustafa Uysal   <uysal@cs.umd.edu>
 *
 */
#ifndef __WORLD_H__
#define __WORLD_H__

#include <stdio.h>
#include <assert.h>
#include <sys/types.h>


/*
 * Miscellaneous Constants
 */
#define OK             0           
#define ERROR         -1         



/* 
 * Boolean Constants
 */
#define TRUE           1
#define FALSE          0



/*
 * Switch Values
 */
#define  ON            1
#define  OFF           0



/*
 * Definitions for Argument Declarations
 */
#if defined(SP2)
#define  ARGS(x)         ()
#else
#define  ARGS(x)         ()
#endif defined(SP2)

#define  IN
#define  IN_OUT
#define  OUT



/*
 * Definitions for Variable Declarations
 *
 */
#define  EXPORT
#define  IMPORT      extern
#define  PRIVATE     
#define  PUBLIC



/*
 * Common Arithmetic Macros 
 */
#if !defined(MIN)
#define MIN(x,y)     (((x) < (y)) ? (x) : (y))
#endif

#if !defined(MAX)
#define MAX(x,y)     (((x) < (y)) ? (y) : (x))
#endif




/*
 * Memory Management
 */

#define Malloc(sz)        safe_malloc(sz, __FILE__, __LINE__)
#define Calloc(ct, sz)    safe_calloc(ct, sz, __FILE__, __LINE__)
#define Free(pt)          safe_free(pt, __FILE__, __LINE__)

extern caddr_t safe_malloc ARGS((int size, char *fn, int ln));
extern caddr_t safe_calloc ARGS((int cnt, int size, char *fn, int ln));
extern void  safe_free ARGS((caddr_t ptr, char *fn, int ln));



/*
 * Macros for code development 
 *
 */
#define message(fmt)      { printf fmt; fflush(stdout); }

#define print_if(exp,fmt) { if (exp) message(fmt); }

#if 0
#define panic(fmt) \
    { printf("\n*Error* in %s at line %d : \n\t", __FILE__, __LINE__);\
      printf fmt; fflush(stdout); exit(1); }
#else
#define panic(fmt) \
    { char *adr = 0; printf("\n*Error* in %s at line %d : \n\t", __FILE__, __LINE__);\
      printf fmt; fflush(stdout); *adr = 0; }
#endif

#define exit_if(exp,fmt)  { if (exp) panic(fmt); }


#define positive(x)  (((x) > 0) ? x : 0);


#define MEGABYTE  0x100000
#define TOOMANY   0x100000

double CLOCK ARGS((void));


#endif __WORLD_H__




