/*
 * list.c
 *
 * $Source: /fs/nene/uysal/pf/newFact/RCS/list.c,v $
 *
 * $Header: /fs/nene/uysal/pf/newFact/RCS/list.c,v 1.1 1995/10/16 03:42:25 uysal Exp $
 *
 * Generic implementation of a list structure using pointers
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */

#include "world.h"
#include "list.h"




/* Routine to create a new list structure */
list_t *ListNew()
{
  list_t *new;

  new = (list_t*) Malloc(sizeof(list_t));
		   
  new->load = 0;
  new->tail = new->head = (listnode_t*) NULL;

  return new;
}





/* Routine to destroy a list structure */
void ListDestroy(lp)
    list_t *lp;
{
  listnode_t *np, *pv;

  /* Traverse the list and destroy whatever you see */
  for (np=lp->head; np != NULL; )
    {
      pv = np;
      np = np->next;
      Free(pv);
    }
  Free(pv);

  /* destroy queue data */
  Free(lp);
}



/* Routine to print all members of the list */
void ListPrintAll(lp, pf, str)
    list_t *lp;
    void (*pf)();
    char *str;
{
  listnode_t *np;

  /* Traverse the list and destroy whatever you see */
  message(("\n\t%s load = %d", str, lp->load));
  for (np=lp->head; np != NULL; np=np->next)
    (*pf)(np->data, str);
}






/* Routine to insert an item into the list at the tail */
void ListInsertTail(lp, dp)
    list_t *lp;
    caddr_t dp;
{
  listnode_t *np;

  np = (listnode_t*) Malloc(sizeof(listnode_t));

  np->next = NULL;
  np->data = dp;

  lp->load++;
  if (lp->tail)
    {
      lp->tail->next = np;
      lp->tail = np;
    }
  else
    {
      lp->head = lp->tail = np;
    }
}





/* Routine to lookup an item in the list */
caddr_t  ListLookup(lp, dp, fn)
    list_t *lp;
    caddr_t dp;
    int (*fn)();
{
  listnode_t *np;

  for (np=lp->head; np != NULL; np = np->next)
    if ((*fn)(dp, np->data) == 0)
      return np->data;

  return NULL;
}





/* Routine to find and delete an item from the list */
caddr_t ListFindDelete(lp, dp, fn)
    list_t *lp;
    caddr_t dp;
    int (*fn)();
{
  listnode_t *np, *pv;

  for (pv=np=lp->head; np!=NULL; )
    if ((*fn)(dp, np->data) == 0)
      {
	if (pv == np)  
	  {
	    lp->head = lp->head->next;
	    if (!lp->head) lp->tail = NULL;
	  }
	else if (lp->tail == np)
	  {
	    lp->tail = pv;
	  }
	
	pv->next = np->next;
	dp = np->data;
	Free(np);
	lp->load--;

	return dp;
      }
    else
      {
	pv = np;
	np = np->next;
      }
  
  return NULL;
}






/* Routine to delete the head of the list */
caddr_t ListDeleteHead(lp)
    list_t *lp;
{
  caddr_t dp;

  if (!lp->head) return NULL;

  lp->load--;
  dp = lp->head->data;
  if (lp->head = lp->tail)
    lp->head = lp->tail = NULL;
  else
    lp->head = lp->head->next;

  return dp;
}




/* Routine to check whether the list is empty or not */
int ListEmpty(lp)
    list_t *lp;
{
  return lp->head ? 0 : 1;
}
