/*
 * snode.h
 *
 * $Source: /fs/nene/uysal/pf/newFact/RCS/snode.h,v $
 *
 * $Header: /fs/nene/uysal/pf/newFact/RCS/snode.h,v 1.1 1995/10/16 03:42:25 uysal Exp $
 *
 * Resource management routines for managing the
 * blocks in a supernode
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#ifndef __SNODE_H__
#define __SNODE_H__

#include "world.h"
#include "block.h"
#include "queue.h"


#define SN_NULL     -1

#define SN_READING   0
#define SN_WRITING   1
#define SN_LOCKWT    3
#define SN_INUSE     4


/*--------------- Data Types ---------------*/

typedef struct snode {
  int sid;         /* supernode number */
  int offset;      /* Location on the local disk */
  int size;        /* size of local block descriptors + data */
  int status;      /* with respect to Location in hierarchy */

  caddr_t buffer;  /* buffer into which the data is stored */
  
  int blkcnt;      /* # of local blocks on processor */
  block_t *blkPtr; /* block descriptors */

  queue_t *bufq;   /* queue of buffers allocated for the supernode */
} snode_t;


/*--------------- Exported Functions ---------------*/

/* Routine to initialize supernode control structure */
void SnodeInit ARGS((int cnt, int *offset, int *size));

/* Routine to extract the dependent supernode list */
int *SnodeUpdaters ARGS((snode_t *sptr, int *cnt));

/* Routine to allocate and initialize mem space */
snode_t *SnodeAlloc ARGS((int id, int offset, int size));

/* Routine to reclaim memory allocated by a supernode */
void SnodeFree ARGS(( snode_t *ptr ));

/* Routine to service a supernode request from memory */
snode_t *SnodeRequest ARGS((int id, int offset, int size, int mode));

#endif  __SNODE_H__

/*
   Note :
   
   STATUS field can assume following values :

   SN_READING
      Asynchronous READ request is in progress

   SN_WRITING
      Asynchronous WRITE request is in progress

   SN_LOCKWT
      SN requested while being written to disk, spfree not allowed

   SN_CACHED
      Contents of this supernode is cached in memory       

   SN_PFETCHD
      Supernode is scheduled to be used in the current burst

   SN_INUSE
      Denotes the current pivot supernode - cannot be flushed out!

   SN_WRITTEN
      Denotes that the supernode write is completed
*/   

