/*
 * dep.c
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/dep.c,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/dep.c,v 1.2 1995/10/09 19:27:01 uysal Exp $
 *
 * Source code to process data dependencies of supernodes to aide
 * in prefetching in the Factorization routines
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include "world.h"
#include "mppworld.h"
#include "queue.h"
#include "dep.h"


#define DEP_BUF_SZ    MEGABYTE

#define DEP_INIT      0
#define DEP_EXTRACT   1
#define DEP_STORE     2


/*--------------- Local Variables of the Module ---------------*/


void IndexRegister ARGS((queue_t *qptr, int index));





/*--------------- Module Implementation ---------------*/


/* Routine to Initialize Local Data Structures for the Dependency Module */
void DepModuleInit()
{
  DepServer(DEP_INIT, 0, 0);
}





/* Routine to be called for distributed dependency processing */
int *DepProcessing(id, map, cnt, ndxq)
    int id, map, *cnt;
    queue_t *ndxq;
{
  int *deplist, depcnt;
  int *retlist, *ptr;
  queue_t *depq;
  
  depq = QueueNew();
  DepServer(DEP_EXTRACT, id, depq);
  DepServer(DEP_STORE, id, ndxq);  
  
  /* Process the dependencies for id */
  deplist = (int*) calloc(depq->load+1, sizeof(int));
  CHKNULL(deplist);
  for (depcnt = 0; !QueueEmpty(depq); depcnt++)
    {
      ptr = (int*) QueueDelete(depq);
      deplist[depcnt] = *ptr;
      free(ptr);
    }

  /* Combine all local dependencies */
  MsgConcat(deplist, depcnt*sizeof(int), &retlist, cnt);
  *cnt /= sizeof(int);

  QueueDestroy(depq);
  return retlist;
}







/* Server routine to keep track of local dependency information */
void DepServer(cmd, snid, ndxq)
    int cmd;
    int snid;
    queue_t *ndxq;
{
  static int *ndxbuf = NULL;
  static int ndxcnt = 0;
  int sid,i,j,prev,*ptr;

  switch(cmd)
    {
    case DEP_INIT:
      ndxbuf = (int*) calloc(DEP_BUF_SZ, sizeof(int));
      CALCHK(ndxbuf, DEP_BUF_SZ, sizeof(int));
      ndxcnt = 0;
      break;

    case DEP_EXTRACT:
      for (sid=i=j=0; i<ndxcnt; )
	{
	  if (ndxbuf[i] == snid)
	    {
	      IndexRegister(ndxq, sid);
	      i++;
	    }
	  else if (ndxbuf[i] == -1)
	    {
	      ndxbuf[j++] = ndxbuf[i++];
	      sid = ndxbuf[i];
	      ndxbuf[j++] = ndxbuf[i++];
	    }
	  else
	    {
	      ndxbuf[j++] = ndxbuf[i++];
	    }
	}
      ndxcnt = j;
      break;

    case DEP_STORE:
      /* do nothing if requested to do nothing */
      if (!ndxq) return;

      /* Flush contents of ndxq into buffer */
      ndxbuf[ndxcnt++] = prev = -1;
      ndxbuf[ndxcnt++] = snid;
      while(!QueueEmpty(ndxq) && ndxcnt < DEP_BUF_SZ)
	{
	  ptr = (int*) QueueDelete(ndxq);
	  if (*ptr != prev)
	    {
	      prev = *ptr;
	      ndxbuf[ndxcnt++] = prev;
	    }
	  free(ptr);
	}
      EXIT_IF(!QueueEmpty(ndxq), (msg, "NDXBUF is too small : %d [short:%d]", ndxcnt, ndxq->load));
      break;

    default:
      PANIC((msg, "Invalid branch cmd=%d", cmd));
    }
}






void IndexRegister(qp, entry)
    queue_t *qp;
    int entry;
{
  int *ptr;

  ptr = (int*) malloc(sizeof(int));
  MALCHK(ptr, sizeof(int));

  *ptr = entry;
  QueueAdd(qp, ptr);
}
