/*
 * io.c
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/io.c,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/io.c,v 1.3 1995/10/09 19:36:08 uysal Exp $
 *
 * Disk I/O module for the parallel block partitioner. This module
 * is responsible for providing support of I/O operations to
 * various components of the block partitioner.
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>

#include "world.h"
#include "queue.h"
#include "io.h"



static queue_t *preadq;  /* queue of pending async read requests */
static queue_t *pwritq;  /* queue of pending async write requests */
static queue_t *outioq;  /* queue of outstanding AIO requests */
extern int errno;


/*--------------- Local Function Prototypes ---------------*/




/*--------------- Implementation ---------------*/					
	

void IOinit()
{
  preadq = QueueNew();
  pwritq = QueueNew();
  outioq = QueueNew();
}


										

/*
 * routine to yield the time taken by a read operation
 */
int TimedRead(fd, buf, size, timer, ctr)
    int fd;
    char *buf;
    int size;
    double *timer, *ctr;
{
  double t0, t1;
  int nb;
  
  t0 = CLOCK();
  nb = read(fd,buf,size);
  t1 = CLOCK();

  EXIT_IF(nb<0, (msg, "File Read Error for size:%d", size));

  *timer += (t1 - t0);
  *ctr   += (double) nb;

  return nb;
}




/*
 * routine to yield the time taken by a seek operation
 */
void TimedSeek(fd, offset, whence, timer, ctr)
    int fd;
    int offset;
    int whence;
    double *timer;
    int *ctr;
{
  double t0, t1;

  t0 = CLOCK();
  lseek(fd, offset, whence);
  t1 = CLOCK();

  *timer += (t1 - t0);
  (*ctr)++;
}




/*
 * routine to yield the time taken by a write operation
 */
void TimedWrite(fd, buf, size, timer, ctr)
    int fd;
    char *buf;
    int size;
    double *timer, *ctr;
{
  double t0, t1;
  int nbw;

  t0 = CLOCK();
  nbw = write(fd,buf,size);
  EXIT_IF(nbw!=size, (msg, "Failed write size:%d nbw:%d errno:%d", size, nbw, errno));
  
  t1 = CLOCK();

  *timer += (t1 - t0);
  *ctr   += (double) size;
}



/*
 * routine to schedule an asynchronous write operation
 */
