/*
 * io.h
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/io.h,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/io.h,v 1.2 1995/10/09 19:27:01 uysal Exp $
 *
 * Useful macros for I/O rate measurments
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#ifndef __IO_H__
#define __IO_H__

#include <aio.h>
#include "world.h"


#define AIO_READ   0
#define AIO_WRITE  1
#define AIO_DONE   2
#define AIO_CONT   3



/*--------------- Exported DataTypes ---------------*/


typedef struct {
  struct aiocb handle;   /* AIO control block */
} aio_t;




/*--------------- Function Prototypes ---------------*/

/* routine to initialize IO module */
void IOinit ARGS((void));

/* routine to clean-up IO module */
void IOexit ARGS((void));

/* routine to yield the time taken by a read operation */
int TimedRead ARGS((int fd, char *buf, int size, double *timer, double *ctr));

/* routine to yield the time taken by a seek operation */
void TimedSeek ARGS((int fd, int offset, int whence, double *timer, int *ctr));

/* routine to yield the time taken by a write operation */
void TimedWrite ARGS((int fd, char *buf, int size, double *timer, double *ctr));

/* routine to issue an async IO request */
aio_t *IssueAIOwrite ARGS((int fd, char *buf, int size, int offset, int whence));

/* routine to issue an async read request */
aio_t *IssueAIOread ARGS((int fd, char *buf, int size, int offset, int whence));

/* routine to check the status of an asynchronous I/O operation */
int AIOstatus ARGS((aio_t *req));

#endif  __IO_H__
