/*
 * main.c
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/main.c,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/main.c,v 1.2 1995/10/09 19:27:01 uysal Exp $
 *
 * Parallel Block Partitioning for Out-of-core Factorization
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */

#include "world.h"
#include "mppworld.h"
#include "datatype.h"
#include "disk.h"

/*--------------- Function Prototypes ---------------*/
void usage();
void GetParameters();
double CLOCK();
void BlockPartition();
int *SupernodeExtract();
void DataTypeInit ARGS((void));


/*--------------- Global Variables ---------------*/
char *progname;    /* Binary executable of program */
nonzero_t ZERO;

/*--------------- Implementation ---------------*/

main(argc,argv)
     int argc;
     char **argv;
{
  char *origfile, *symbfile, *outfile;
  int  *sndNdx, sndCnt;   
  double bp_time, se_time, t0, t1;
  int sfid;
  
  /* Initialization */
  MPPinit();
  bp_time = se_time = 0.0;
  
  /* Parameter processing */
  GetParameters(argc, argv, &origfile, &symbfile, &outfile);
  DataTypeInit();
  DiskTimingInit();
  
  if (!MyProc())
    {
      printf("\nParallel Block Partitioner\n");
      printf("\n\tOriginal Matrix File : %s", origfile);
      printf("\n\tSymbolic Factor      : %s", symbfile);
      printf("\n\tOutput File          : %s", outfile);
      printf("\n\tNonzero size         : %d", sizeof(nonzero_t));
      printf("\n");
    }
  
  /* Supernode extraction from the signature of SymFact */
  t0 = CLOCK();
  sndNdx = SupernodeExtract(symbfile, &sndCnt, &sfid);
  t1 = CLOCK();
  se_time = (t1 - t0);

  printf("\nSN extract done, %d Supernodes", sndCnt);

  /* Fill Processing and Block Decomposition */
  t0 = CLOCK();
  BlockPartition(sndCnt, sndNdx, sfid, origfile, outfile);
  t1 = CLOCK();
  bp_time = (t1 - t0);

  printf("\n\tSE:[%.3lf secs] BP:[%.3lf secs]\n", se_time, bp_time);
  if (!MyProc()) printf("\n\n");
}






/*
 * GetParameters()
 *
 * Process the command line parameters and set the working mode
 * variables for the main processing unit 
 *
 */
void GetParameters(argc, argv, matptr, symptr, outptr)
    int argc;
    char *argv[];
    char **matptr, **symptr, **outptr;
{
  char *path;
  int  argndx, i;

  /* input control police */
  if (argc<4) usage();

  /* get the executables' name */
  progname = argv[0];

  /* input file */
  argndx = 1;
  *matptr = argv[argndx++];
  *symptr = argv[argndx++];  
  *outptr = argv[argndx];
}







/*
 * usage()
 *
 * Early exit from the program with a display of program usage
 *
 */
void usage()
{
  fprintf(stderr, "\n\n");
  fprintf(stderr, "\nUsage : %s MatrixFile SymFactFile outputFile\n", progname);
  fprintf(stderr, "\n\n");
  exit(1);
}



void DataTypeInit()
{
  fp_t fzero;

  fzero = (fp_t) 0.0;

#if   defined(COMPLEX)
  ZERO.RealPart = ZERO.ImagPart = fzero;
#elif defined(REAL)
  ZERO = fzero;
#else
#error "One of COMPLEX or REAL should be defined"
#endif
}
