/*
 * mem.c
 *
 * $Source: /fs/nene/uysal/pf/newFact/RCS/mem.c,v $
 *
 * $Header: /fs/nene/uysal/pf/newFact/RCS/mem.c,v 1.1 1995/10/16 03:42:25 uysal Exp $
 *
 * Memory management module
 *
 * Author : Mustafa Uysal (uysal@cs.umd.edu)
 *
 */
#include <malloc.h>
#include "mem.h"

#include "nworld.h"


/* 32 Meg Available for Data */
#define MEM_CAPACITY  0x2000000   

static int mem_avail;




void MemInit(verbose)
    int verbose;
{
  mem_avail = MEM_CAPACITY;
  if (verbose) printf("In-core memory size = %d", mem_avail);
}



void MemExit()
{
  MemStatus();
}



void MemStatus()
{
  message(("\tMEM begin:%d end:%d", MEM_CAPACITY, mem_avail));
}



caddr_t MemGet( size )
     int size;
{
  caddr_t new;

  if (mem_avail < size) return (caddr_t) NULL;
  mem_avail -= size;
  new = (caddr_t) Malloc(size);
  
  return new;
}





void MemPut(ptr, size)
     caddr_t ptr;
     int     size;
{
#if 0
  if (!(size>0)) * ((int*)size) = 0; /* SEGV */
  EXIT_IF(!(size>0), (msg, "MemPut - Invalid request : %d", size));
  EXIT_IF(!(mem_avail>=0), (msg, "MemPut - Invalid entry state : %d", mem_avail));
#endif  

  mem_avail += size;
  Free(ptr);
}






caddr_t safe_malloc(size, fn, ln)
    int size;
    char *fn;
    int ln;
{
  caddr_t ptr;

  if (!(ptr = malloc(size)))
    panic(("malloc(%d) in %s at line %d failed.", size, fn, ln));

  return ptr;
}





caddr_t safe_calloc(cnt, size, fn, ln)
    int cnt;
    int size;
    char *fn;
    int ln;
{
  caddr_t ptr;

  if (!(ptr = calloc(cnt, size)))
    panic(("calloc(%d,%d) in %s at line %d failed.", cnt, size, fn, ln));

  return ptr;
}



void  safe_free(ptr, fn, ln)
    caddr_t ptr;
    char *fn;
    int ln;
{
  exit_if(!ptr, ("Trying to free a null pointer [Line %d of %s]", ln, fn));
  free(ptr);
}

