/*
 * mppworld.h
 *
 * $Source: /fs/nene/uysal/source/RCS/mppworld.h,v $
 *
 * $Header: /fs/nene/uysal/source/RCS/mppworld.h,v 1.2 1995/09/29 01:12:12 uysal Exp $
 *
 * Generic header file for MPP related functions and utilities
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#ifndef __MPPWORLD_H__
#define __MPPWORLD_H__

#include "world.h"

#if    defined(SP2)
#include "mpproto.h"
#endif defined(SP2)

/*--------------- Blocking Communication Routines ---------------*/

/* Routine to send a blocking message */
void MsgSend ARGS((IN int tag, IN int to, IN char *buf, IN int len));

/* Routine to recv a message (blocking) */
int  MsgRecv ARGS((IN int tag, IN char *buf, IN int len));

/* Routine to recv a message from a particular processor (blocking) */
int  MsgRecvFrom ARGS((IN int tag, IN char *buf, IN int len, IN int src));

/* Routine to recv all pending messages and clear incoming msg queue */
void MsgRecvDiscardAll ARGS(( void ));

/* Routine to discard a message with a given tag */
void MsgDiscard ARGS((int tag));

/* Routine to probe the incoming message queue */
int MsgProbe ARGS((int tag, int from));

/*--------------- Asynchronous Communication Routines ---------------*/

/* Routine to send an asynchronous message */
void CommAsyncSend ARGS((caddr_t adr, int to, int tag, int size));

/* Routine to recv an asynchronous message */
void CommAsyncRecv ARGS((caddr_t adr, int from, int tag, int size));

/* Routine to poll the recv queue to get a message */
caddr_t RecvQueuePoll ARGS((int *sz, int *fm, int *tg));

/* Routine to poll the send queue and return the address of a completed message */
caddr_t SendQueuePoll ARGS((void));


/* Routine to Probe the send queue */
int SendQueueProbe ARGS((void));

/* Routine to drain the send queue */
void SendQueueDrain ARGS((void));

/* Routine to drain the send queue */
void SendQueueFlush ARGS((void));

/* Routine to report the status of async communications */
void CommAsyncStatus ARGS((void));

/* Routine to probe the incoming message queue for completions */
caddr_t RecvQueueProbe ARGS((int *sz, int *fm, int *tg));

/*--------------- Collective Routines ---------------*/

/* Routine to broadcast a message to all nodes */
void MsgBroadcast ARGS((int origin, void *message, int size, int tag));

/* Routine to synchronize all processors */
void Sync ARGS((void));

/* Routine to get the id of the processor on the left in a ring */
int LeftProc ARGS((int proc));

/* Routine to get the id of the processor on the right in a ring */
int RightProc ARGS((int proc));

/* Routine to check if the proc is the RightMost processor in a ring */
int LastProc ARGS((int proc));

/* Routine to concatenate local buffers in processor order */
void MsgConcat ARGS((char *inbuf, int inlen, char **outbuf, int *outlen));


/*--------------- Miscellaneous Routines ---------------*/

/* Routine to access the local clock */
double CLOCK ARGS((void));

/* Routine to query the number of processors */
int NumProcs ARGS((void));

/* Routine to get the local processor id */
int MyProc ARGS((void));

/* Routine to initialize the MPP module */
void MPPinit ARGS((void));

#if defined(DRAINER)
int NetworkDrain ARGS((void));
#endif defined(DRAINER)

#endif  __MPPWORLD_H__


