/*
 * sp2.c
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/sp2.c,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/sp2.c,v 1.3 1995/10/09 19:36:08 uysal Exp $
 *
 * Implementation of configuration and message passing functions for
 * the IBM SP-2 platform.
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include <mpproto.h>

#if    !defined(SP2)
#define SP2
#endif !defined(SP2)

#include "world.h"
#include "mppworld.h"
#include "queue.h"


#define MSG_CONCAT  40000

/*-------------- Type Definitions ----------------------------*/

/* sendq member item type */
typedef struct {
  int      msgid;    /* msg identifier for the async send */
  caddr_t  bufadr;   /* ptr to the message buffer */
} send_t;


/* recvq member item type */
typedef struct {
  caddr_t  buffer;
  int      msgid;
  int      source;
  int      tag;
} recv_t;


/*-------------- Global Variable Declarations ----------------*/

static queue_t *sendq; /* queue of outstanding asynchrounous send requests */
static queue_t *recvq; /* queue of outstanding asynchrounous recv requests */
static int me, np;



/*-------------- Local Function Prototypes ---------------------*/
double w_seconds ARGS((void));



/*--------------- Implementation ---------------*/


int NumProcs( VOID )
{
  int numtask, taskid;

  mpc_environ(&numtask, &taskid);
  return(numtask);
}




int MyProc( VOID )
{
  int numtask, taskid;

  mpc_environ(&numtask, &taskid);
  return(taskid);  
}



void MPPinit()
{
  me = MyProc();
  np = NumProcs();

  /* initialize async send/recv message queues */
  sendq = QueueNew();
  recvq = QueueNew();
} 




/*
 * Communication module status report (for async messages) 
 */
void CommAsyncStatus()
{
  queue_t *tmpq;
  recv_t *rcv;

  MESSAGE((msg, "\tComm: sendq=%d recvq=%d", sendq->load, recvq->load));
  
  if (!QueueEmpty(recvq))
    {
      tmpq = QueueNew();
      while (!QueueEmpty(recvq))
	{
	  rcv = (recv_t*) QueueDelete(recvq);
	  MESSAGE((msg, "[RCV - %d %d %d]",  rcv->tag, rcv->source, rcv->msgid));
	  QueueAdd(tmpq, rcv);
	}
      QueueDestroy(recvq);
      recvq = tmpq;
    }
}





/* Routine to send an asynchronous message */
void CommAsyncSend(adr, to, tag, size)
    caddr_t adr;
    int     to,tag,size;
{
  send_t *sqitem;
  
  sqitem = (send_t*) malloc(sizeof(send_t));
  CHKNULL(sqitem);
  
  if (mpc_send(adr, size, to, tag, &(sqitem->msgid))!=0)
    {
      PANIC((msg, "Send error : %d [sz:%d to:%d tg:%d]", mperrno, size, to, tag));
    }

#if 0
  else
    {
      MESSAGE((msg, "Send Activate - tg:%d to:%d sz:%d", tag, to, size));
    }
#endif

  sqitem->bufadr = adr;
  QueueAdd(sendq, sqitem);
}



/* Routine to recv an asynchronous message */
void CommAsyncRecv(adr, from, tag, size)
    caddr_t adr;
    int     from,tag,size;
{
  recv_t *rqitem;
  
  rqitem = (recv_t*) malloc(sizeof(recv_t));
  CHKNULL(rqitem);

  rqitem->buffer = adr ? adr : (caddr_t) malloc(size);
  CHKNULL(rqitem->buffer);

  rqitem->source = from;
  rqitem->tag    = tag;
  if (mpc_recv(rqitem->buffer, size, &(rqitem->source), &(rqitem->tag), &(rqitem->msgid))!=0)
    {
      PANIC((msg, "Recv error : %d [sz:%d fm:%d tg:%d]", mperrno, size, from, tag));
    }
  /*
  else
    {
      MESSAGE((msg, "Recv Activate - tg:%d fm:%d sz:%d", tag, from, size));
    }
  */
  
  QueueAdd(recvq, rqitem);
}



/* Routine to poll the outgoing message queue for completions */
caddr_t SendQueuePoll()
{
  caddr_t adr;
  send_t *snd;
  int cnt;

  if (QueueEmpty(sendq))
    return (caddr_t) NULL;

  snd = (send_t*) QueueDelete(sendq);
  for (cnt=0; mpc_status(snd->msgid)<0; cnt++)
    if (cnt>=(TOOMANY*5)) PANIC((msg, "SendQueuePoll() blocked."));

  adr = snd->bufadr;
  free(snd);

  return adr;
}




/* Routine to drain the send queue */
void SendQueueDrain()
{
  send_t *snd;
  int cnt;

  while (!QueueEmpty(sendq))
    {
      snd = (send_t*) QueueDelete(sendq);
      for (cnt=0; mpc_status(snd->msgid)<0; cnt++)
	if (cnt>=(TOOMANY))
	  PANIC((msg, "SendQueueDrain() blocked."));
      free(snd);
    }
}



/* Routine to poll the incoming message queue for completions */
caddr_t RecvQueuePoll()
{
  caddr_t adr;
  recv_t *rcv;
  int cnt = 0;
  
  if (QueueEmpty(recvq))
    return NULL;

  rcv = (recv_t*) QueueDelete(recvq);
  for (cnt=0; mpc_status(rcv->msgid)<0; cnt++)
    {
      if (cnt>(10*TOOMANY))
	{
	  MsgRecvDiscardAll();
	  PANIC((msg, "RecvQueuePoll() blocked for tag:%d from:%d",
		 rcv->tag, rcv->source));
	}
    }
  
  adr = rcv->buffer;
  free(rcv);
  
  return adr;
}




/*
 * Broadcast of a message from a designated processor to
 * all other peers
 */
void MsgBroadcast(origin, message, size, tag)
    int origin;
    void *message;
    int size, tag;
{
  int i;

  if (me != origin)
    {
      MsgRecv(tag, message, size);
      return;
    }
  
  for (i=0; i<np; i++)
    if (i != me)
      MsgSend(tag, i, message, size);
}




void MsgSend(tag, to, buf, len)
     int tag;
     int to;
     char *buf;
     int len;
{
  if (mpc_bsend(buf, len, to, tag) < 0)
    PANIC((msg, "Error in msg send code=%d tag:%d to:%d len:%d", mperrno, tag, to, len));
}





int MsgRecv(tag, buf, len)
     int tag;
     char *buf;
     int len;
{
  int newlen, lsrc, ltag, cnt, lid;

  /* Post an ASYNC Recv */
  lsrc = DONTCARE;
  ltag = tag;
  if (mpc_recv(buf, len+MEGABYTE, &lsrc, &ltag, &lid)!=0)
    {
      PANIC((msg, "Recv error : %d [sz:%d fm:%d tg:%d]", mperrno, len, lsrc, tag));
    }

  /* Block for it */
  for (cnt=0; (newlen = mpc_status(lid))<0; cnt++);
#if 0
    if (cnt>=(TOOMANY*5))
      {
	ltag = lsrc = DONTCARE;
	mpc_probe(&lsrc, &ltag, &newlen);
	MESSAGE((msg, "Result of WildCard probe t:%d s:%d l:%d", ltag, lsrc, newlen));
	PANIC((msg, "MsgRecv() blocked for tag:%d frm:%d siz:%d", tag, lsrc, len));
      }
#endif

  /* after op checks */
  EXIT_IF(newlen>len,(msg,"ShortBuffer tag=%d len=%d nlen=%d src=%d",
		      tag, len, newlen, lsrc));
  
  return newlen;
}





int MsgRecvFrom(tag, buf, len, src)
     int tag;
     int src;
     char *buf;
     int len;
{
  int newlen, lsrc, ltag, cnt, lid;

  /* Post an ASYNC Recv */
  lsrc = src;
  ltag = tag;
  if (mpc_recv(buf, len+MEGABYTE, &lsrc, &ltag, &lid)!=0)
    {
      PANIC((msg, "Recv error : %d [sz:%d fm:%d tg:%d]", mperrno, len, src, tag));
    }

  /* Block for it */
  for (cnt=0; (newlen = mpc_status(lid))<0; cnt++)
    if (cnt>=(TOOMANY))
      {
	ltag = lsrc = DONTCARE;
	mpc_probe(&lsrc, &ltag, &newlen);
	MESSAGE((msg, "Result of WildCard probe t:%d s:%d l:%d", ltag, lsrc, newlen));
	PANIC((msg, "MsgRecvFrom() blocked for tag:%d frm:%d siz:%d", tag, src, len));
      }

  /* after op checks */
  EXIT_IF(newlen>len,(msg,"ShortBuffer tag=%d len=%d nlen=%d src=%d",
		      tag, len, newlen, src));
  
  return newlen;
}




void MsgRecvDiscardAll()
{
  char *tbuf;
  int llen, lsrc, ltag, cnt;
  int newlen;
  
  ltag = lsrc = DONTCARE;
  while(mpc_probe(&lsrc, &ltag, &newlen)==0)
    {
      if (newlen<0) break;
      MESSAGE((msg, "Discard tg:%d fm:%d ln:%d", ltag, lsrc, newlen));
      tbuf = (char*) malloc(newlen+1);
      CHKNULL(tbuf);
      mpc_brecv(tbuf, newlen, &lsrc, &ltag, &llen);
      free(tbuf);
      ltag = lsrc = DONTCARE;
    }
}



double CLOCK()
{
  return w_seconds();
}




void Sync( void )
{
  mpc_sync(ALLGRP);
}


int LeftProc( proc )
    int proc;
{
  return (!proc) ? (np-1) : (proc-1);
}



int RightProc( proc )
    int proc;
{
  return (proc==(np-1)) ? 0 : proc+1;
}


int LastProc( proc )
    int proc;
{
  return (proc==(np-1)) ? 1 : 0;
}




/* Routine to concatenate local buffers in processor order */
void MsgConcat(inbuf, inlen, outbuf, outlen)
    char *inbuf, **outbuf;
    int inlen, *outlen;
{
  char *lclbuf;
  int *szvct, *gsvct, tsize;
  int *srcvct, *typvct, *idvct;
  int i,j;

  /* default return values */
  *outbuf = NULL;
  *outlen = 0;

  /* prepare comm buffers */
  szvct = (int*) calloc(np, sizeof(int));
  CALCHK(szvct, np, sizeof(int));
  for (i=0; i<np; i++)
    szvct[i] = 0;
  szvct[me] = inlen;
  gsvct = (int*) calloc(np, sizeof(int));
  CALCHK(gsvct, np, sizeof(int));

  /* obtain global size vector */
  mpc_combine(szvct, gsvct, np*sizeof(int), i_vadd, ALLGRP);

  /* allocate memory for output */
  for (tsize=i=0; i<np; i++)
    tsize += gsvct[i];

  if (!tsize) return;

  lclbuf = (char*) malloc(tsize);
  MALCHK(lclbuf, tsize);
  *outlen = tsize;
  
  /* post local rcvs from each processor - crude implementation : ALL-to-ALL */
  srcvct = (int*) calloc(np, sizeof(int));
  CALCHK(srcvct, np, sizeof(int));
  typvct = (int*) calloc(np, sizeof(int));
  CALCHK(typvct, np, sizeof(int));
  for (tsize=i=0; i<np; i++)
    if (i!=me)
      {	
	srcvct[i] = i;
	typvct[i] = MSG_CONCAT+i;
	mpc_recv(lclbuf+tsize, gsvct[i], &srcvct[i], &typvct[i], &szvct[i]);
	tsize += gsvct[i];
	mpc_send(inbuf, inlen, i, MSG_CONCAT+me, &gsvct[i]);
      }
    else
      {
	memcpy(lclbuf+tsize, inbuf, inlen);
	tsize += gsvct[i];
      }

  /* Wait for messages to complete */
  for (i=0; i<np; i++)
    if (i!=me)
      {
	mpc_wait(&szvct[i], &tsize);
	mpc_wait(&gsvct[i], &tsize);
      }

  free(szvct);
  free(gsvct);
  free(srcvct);
  free(typvct);

  *outbuf = lclbuf;
}



		  


