/*
 * supernode.c
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/supernode.c,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/supernode.c,v 1.2 1995/10/09 19:27:01 uysal Exp $
 *
 * Routines to identify the supernodes in a symbolic factor
 * stored only as row indices (ie, output of symfact).
 * Symbolic factor is read in large chunks.
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include <fcntl.h>
#include <malloc.h>

#include "world.h"
#include "mppworld.h"
#include "io.h"
#include "mtags.h"

#define CHUNK_SIZE  (25*MEGABYTE)


/*--------------- External Timing Data ---------------*/
double sn_rt, sn_br;
double sn_time, sm_time;
  

/*--------------- Local Function Prototypes ---------------*/

/* routine to compare two index sets */
int IndexSetCompare ARGS((int *set1, int *set2));

/* routine to merge and duplicate local separators in all of the processors */
int *MergeSeparators ARGS(( int *list, int size, int *outsize, int *temp));

/* routine to merge two separator sets */
int *LocalMerge ARGS(( int *set1, int *size1, int *set2, int *size2));



/*--------------- Module Implementation ---------------*/


int *SupernodeExtract(fname, scnt, sfid)
    char *fname;
    int  *scnt;
    int  *sfid;
{
  int *slist;          /* list of supernode separating column ids */
  int fid;             /* file identifier */
  int shiftcnt;        /* block epoch counter */
  int *buf, *ghostbuf; /* ptrs to locally allocated storage */
  int offset;         
  int nc, csiz, nsiz, ndx;
  int *border, *ghostptr;
  int chunksz, reqsz, sreqsz, schnsz;
  int firsttime;
  int flag, lcnt;
  double t0, t1;
  int rtg, rfm, rsz;

  fid = open(fname, O_RDONLY);
  EXIT_IF(fid<0, (msg, "Error in opening file : %s", fname));

  sn_br = sn_rt = sn_time = sm_time = 0.0;
  buf = (int*) malloc(CHUNK_SIZE);
  MALCHK(buf, CHUNK_SIZE);

  flag = firsttime = 1;
  offset=lcnt=0;
  shiftcnt = 0;
  reqsz = CHUNK_SIZE;
  border = NULL;
  
  do {
    chunksz = TimedRead(fid, buf+offset/sizeof(int), reqsz, &sn_rt, &sn_br);
    sreqsz = reqsz;
    schnsz = chunksz;
    
    t0 = CLOCK();
    if (firsttime)
      {
	nc = buf[0];	
	firsttime = 0;
	ndx = 3;
	ghostbuf = (int*) calloc(nc, sizeof(int));
	CHKNULL(ghostbuf);
	slist = (int*) calloc(nc, sizeof(int));
	CHKNULL(slist);
	if (MyProc())
	  CommAsyncSend(&buf[ndx], LeftProc(MyProc()), MSG_SHIFT+shiftcnt, buf[ndx]*sizeof(int));
	else
	  slist[lcnt++] = 1;

	CommAsyncRecv(ghostbuf, RightProc(MyProc()),
		      MSG_SHIFT+shiftcnt+LastProc(MyProc()), nc*sizeof(int));
      }
    else ndx = 0;

    /* supernode extract process happens in rounds of blocks */
    for (chunksz+=offset, offset=0; (ndx*sizeof(int)) < chunksz; ndx += csiz)
      {
	/*MESSAGE((msg, "\tC:%d L:%d N:%d", buf[ndx+1], buf[ndx], ndx));*/

	csiz = buf[ndx];        /* size of the current column */
	nsiz = buf[ndx+csiz];   /* size of the next column */

	if (nsiz<=0||buf[ndx+1]==nc||buf[ndx+1]<0) break;
	    
	/* getout if the remaining data is NOT TWO whole columns */
	if (!((ndx+csiz+nsiz)*sizeof(int)<chunksz))
	  {
	    SendQueueDrain();
	    offset = chunksz-ndx*sizeof(int);  /* #bytes to copy */
	    memcpy(buf, &buf[ndx], offset);
	    reqsz = CHUNK_SIZE - offset;       /* adjust request size */
	    assert((offset%4)==0);
	    break;
	  }

	/*lastcol = buf[ndx+1];*/

	/* is the column at the block boundary? */
	if ((buf[ndx+1]!=(buf[ndx+csiz+1]-1)))
	  {
	    shiftcnt++;

	    /* remember the last column of current block */
	    border = &buf[ndx];

	    /*
	    if ((buf[ndx+csiz+1])>44000)
	      MESSAGE((msg,"CASend(c:%d t:%d d:%d l:%d=%d)", buf[ndx+csiz+1],MSG_SHIFT+shiftcnt, 
		       LeftProc(MyProc()), nsiz, buf[ndx+csiz]));
	    */

	    /* send the first column of the NEXT block to my left neighbor */
	    if (buf[ndx+csiz+1]>0)
	      CommAsyncSend(&buf[ndx+csiz], LeftProc(MyProc()),
			    MSG_SHIFT+shiftcnt, nsiz*sizeof(int));
	  }

	/* compare current column with the NEXT */
	if (border)
	  {
	    ghostptr = (int*) RecvQueuePoll();
	    /*
	    if ((MSG_SHIFT+shiftcnt+LastProc(MyProc()))>210)
	      MESSAGE((msg,"CARecv got-(c:%d f:%d t:%d) exp-(c:%d t:%d)",
		       ghostptr[1],border[1],MSG_SHIFT+shiftcnt+LastProc(MyProc())-1,
		       ghostptr[1]+NumProcs()*100,MSG_SHIFT+shiftcnt+LastProc(MyProc())));
	    */

	    EXIT_IF((ghostptr[1]-1)!=border[1], 
		    (msg, "Communication Mismatch got:%d for:%d shf:%d",
		     ghostptr[1], border[1], MSG_SHIFT+shiftcnt+LastProc(MyProc())-1));

	    if (IndexSetCompare(border, ghostptr) == FALSE)
	      slist[lcnt++] = ghostptr[1];
	    if ((ghostptr[1]+NumProcs()*100)<=nc)
	      CommAsyncRecv(ghostbuf, RightProc(MyProc()),
			    MSG_SHIFT+shiftcnt+LastProc(MyProc()), nc*sizeof(int));
	    border = NULL;
	  }
	else if (IndexSetCompare(&buf[ndx], &buf[ndx+csiz]) == FALSE)
	  {
	    slist[lcnt++] = buf[ndx+csiz+1];
	  }
      }/*for*/

    SendQueueDrain();
    t1 = CLOCK();
    sn_time += (t1-t0);

  } while (schnsz == sreqsz);

  if (!MyProc()) slist[lcnt++] = nc+1;

  /*CommAsyncStatus();*/
  
  t0 = CLOCK();
  slist = MergeSeparators(slist, lcnt, scnt, ghostbuf);
  t1 = CLOCK();
  sm_time = (t1 - t0);

  /*CommAsyncStatus();*/

  *sfid = fid;
  free(buf);

  return slist;
}





/*
 * Function to compare two index sets. It is used to
 * determine whether two columns belong to the same supernode
 * or not.
 */
int IndexSetCompare(set1, set2)
    int *set1;
    int *set2;
{
  int  nset1, nset2;
  int  i,j,k;

  /* one of the index sets are NULL */
  if (!set1 || !set2)
    return FALSE;

  /* to be removed later */
  /*EXIT_IF(set1[1]!=(set2[1]-1), (msg, "Invalid comparison c1:%d c2:%d", set1[1], set2[1]));*/
  
  /* Get the index vector sizes */
  nset1 = set1[0];
  nset2 = set2[0];

  /* not identical based on size info */
  if ((nset1 - 1) != nset2)
    return FALSE;

  /* need to compare sets to determine the structural identity */
  for (i=2, j=1; i<nset1; i++, j++)
    if (set1[i] != set2[j])
      return FALSE;

  return TRUE;
}




/*
 * Function to merge and duplicate local supernode
 * separators in all of the processors
 */
int *MergeSeparators(local, lsize, outsize, temp)
    int *local;
    int  lsize;
    int *outsize;
    int *temp;
{
  int *list, cnt;
  int *rlst, rct;
  int  i;

  /* fan-in: all of the lists go to proc 0 */
  if (MyProc())
    {
      MsgSend(MSG_SND_CNT, 0, &lsize, sizeof(int));
      MsgSend(MSG_SND_LST, 0, local, lsize*sizeof(int));
    }
  else
    {
      list = local;
      cnt  = lsize;
      for (i=1; i<NumProcs(); i++)
	{
	  MsgRecvFrom(MSG_SND_CNT, &rct, sizeof(int), i);

	  rlst = (int*) calloc(rct, sizeof(int));
	  CHKNULL(rlst);

	  MsgRecvFrom(MSG_SND_LST, rlst, rct*sizeof(int), i);
	  
	  list = LocalMerge(list, &cnt, rlst, &rct, temp);
	}
    }

  /* fan-out: everybody receives the whole list */
  if (MyProc())
    {
      free(local);

      MsgRecvFrom(MSG_SND_CNT, &cnt, sizeof(int), 0);

      list = (int*) calloc(cnt, sizeof(int));
      CHKNULL(list);

      MsgRecvFrom(MSG_SND_LST, list, cnt*sizeof(int), 0);
    }
  else
    {
      for (i=1; i<NumProcs(); i++)
	{
	  MsgSend(MSG_SND_CNT, i, &cnt, sizeof(int));
	  MsgSend(MSG_SND_LST, i, list, cnt*sizeof(int));
	}
    }

  *outsize = cnt-1;
  return list;
}






int *LocalMerge(set1, cnt1, set2, cnt2, temp)
    int *set1, *cnt1, *set2, *cnt2, *temp;
{
  int *newset;
  int  i,j,k;
  int  nset1, nset2;

  /* Index Set Merge */
  nset1 = *cnt1;
  nset2 = *cnt2;

  for (i=j=k=0; (i<nset1) && (j<nset2); k++)
    {
      if (set1[i] < set2[j])
	temp[k] = set1[i++];
      else if (set1[i] > set2[j])
	temp[k] = set2[j++];
      else
	{
	  temp[k] = set1[i];
	  j++; i++;
	}
    }
  for ( ; i<nset1; )
    temp[k++] = set1[i++];
  for ( ; j<nset2; )
    temp[k++] = set2[j++];

  /* free up previous lists */
  free(set1);
  free(set2);
  
  /* memory allocation */
  newset = (int*) calloc(k, sizeof(int));
  CHKNULL(newset);

  /* copy of new index set into new location */
  for (i=0; i<k; i++)
    newset[i] = temp[i];

  /* the new supernode list is returned here */
  *cnt1 = k;
  return newset;  
}
