//
// buffer.h
//
// Quick and dirty hack for a dynamically extensible buffer
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
#if !defined(__BUFFER_H__)
#define __BUFFER_H__

#include <iostream.h>
#include <assert.h>
#include "vector.h"

class Buffer : public Vector {
  int nused;

public:
  Buffer(void) : Vector(5,-1)
  {
    nused = 0;
  };

  Buffer(int size) : Vector(size,-1)
  {
    nused = 0;
  };

  Buffer(Buffer &base) : Vector(base.nused, base.getData())
    {
      nused = base.nused;
    };

  ~Buffer(void)
  {
    nused = 0;
  }


  int numUsed( void )
  {
    return nused;
  }

  // append an integer into the buffer
  // expand the storage if full
  void append(int elem)
  {
    int vsize = this->getSize();

    //cout << " append: <" << nused << ", " << vsize << "> - elem:" << elem << "\n";

    assert( nused <= vsize );
    if (nused == vsize)
      this->expand(vsize);

    assert( nused < this->getSize());
    this->setElem(nused, elem);
    nused++;
  };


  void purge(void)
  {
    this->Vector::purge();
    nused = 0;
  }

  void merge(int cnt, int *data)
  {
    nused = this->Vector::merge(cnt, nused, data);
  }

  void print(void)
  {
    int vs = this->getSize();
    cout << " size = " << vs << " nused = " << nused << "\n";

    //cout << " first = " << this->getElem(0) << " last = " << this->getElem(nused-1) 
    // << " (total of " << nused << " entries).\n";
  }

  void print(int numInLine, char *msg = NULL)
  {
    for (int k = 0; k < nused; k += numInLine)
      {
	if (msg) cout << msg << ": ";
	for (int j = 0; (j < numInLine) && (k+j < nused); j++)
	  cout << " " << this->getElem(k+j);
	cout << "\n";
      }
  }
};

#endif
