//
// mapping.h
//
// Data/Computation Mapping onto processors
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
//
#if !defined(__MAPPING_H__)
#define __MAPPING_H__

#include <math.h>
#include "mpapi.h"
#include "vector.h"

class TorusMap : public mpapi {
  int nxdim, nydim;   

public:
  TorusMap( void )
    {
      nxdim = nydim = 0;
    }

  TorusMap( mpapi &mp )
    {
      me = mp.myProc();
      np = mp.numProcs();
      
      nydim = (int) sqrt((double)np);
      nxdim = np / nydim;
    }

  int map( int x, int y )
    {
      // cout << "\tMap(" << x << ", " << y << ") = " << mv << ".\n";
      return (((x % nxdim) * nydim) + (y % nydim)) % np; 
    }

  int rowSize( void )
    {
      return nxdim;
    }

  int colSize( void )
    {
      return nydim;
    }

  int isLocal( int col, int row )
    {
      return (this->map(col, row) == me) ? 1 : 0;
    }

  void integrityCheck(void)
    {
      cout << " Torus Map Structure (nxdim = " << nxdim << ", nydim = ";
      cout << nydim << "): [np = " << np << "]\n";
      for (int k = 0; k < nxdim; k++)
	for (int j = 0; j < nydim; j++)
	  cout << "\t(" << k << ", " << j << ") :-> " << this->map(k,j) << ".\n";
    }

  int isRowMember( int row )
    {
      for (int k = 0; k < nxdim; k++)
	if (this->map(k, row) == me) return 1;
      return 0;
    }

  int isColMember( int col )
    {
      for (int k = 0; k < nydim; k++)
	if (this->map(col, k) == me) return 1;
      return 0;
    }

  //Vector procRow( int row )
  //  {
  //    Vector pv(nxdim, -1);
  //    for (int k = 0; k < nxdim; k++)
  //	pv.setElem(k, this->map(k, row));
  //   return pv;
  //  }

  Vector *procCol( int col )
    {
      Vector *pv = new Vector(nydim, -1);
      for (int k = 0; k < nydim; k++)
	pv->setElem(k, this->map(col, k));
      return pv;
    }

  void printCol( int col )
    {
      Vector *colv = this->procCol(col);
      colv->print("MapVector");
    }
};

#endif
