//
// sp.h
//
// Implementation of Message Passing API for IBM SP Systems.
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
//
# if !defined(__SP_H__)
#define __SP_H__

#include <mpproto.h>
#include <iostream.h>
#include "mpapi.h"

#include "vector.h"
#include "buffer.h"

class ibmsp : public mpapi {
public:
  ibmsp( void )
    {
      mpc_environ(&np, &me);
    };

  void init(void)
    {
      mpc_environ(&np, &me);
    };

  int asend(int to, int tag, int size, void *buf)
    {
      int tmp;
      mpc_send(buf, size, to, tag, &tmp);
      return tmp;
    }

  int arecv(int &tag, int &src, int size, void *buf)
    {
      int mid;
      mpc_recv(buf, size, &src, &tag, &mid);
      return mid;
    }

  int mwait(int id)
    {
      size_t size;
      mpc_wait(&id, &size);
      return (int) size;
    }

  void bsend(int to, int tag, int size, void *buf)
    {
      // cout << "\tsend to:" << to << " tag: " << tag << "\n";
      mpc_bsend(buf, size, to, tag);
    }

  void brecv(int &tag, int &src, int &size, void *buf)
    {
      size_t sv   = size;
      int    lsrc = src;
      int    ltag = tag;

      // cout << "\trecv fm:" << src << " tag: " << tag << "\n";
      mpc_brecv(buf, sv, &lsrc, &ltag, &sv);
      size = sv;
      src  = lsrc;
      tag  = ltag;
    }

  void broadcast(int root, int tag, int size, void *buf)
    {
      if (me == root)
	{
	  for (int k = 0; k < np; k++)
	    if (k != me)
	      this->bsend(k, tag, size, buf);
	}
      else
	{
	  this->brecv(tag, root, size, buf);
	}
    }

  void multicast(Vector *set, int tag, int size, void *buffer)
    {
      //cout << "\tMultiCast(" << me << ", " << tag << ") ";
      // set->print("Target");
      for (int k = 0; k < set->getSize(); k++)
	if (set->getElem(k) != me)
	  this->bsend(set->getElem(k), tag, size, buffer);
    }

  void bufCollect(Buffer &mybuf, int tagbase)
    {
      Buffer tbuf(mybuf);
      for (int k = 0; k < np; k++)
	if (me == k)
	  {
	    int size = tbuf.numUsed();
	    this->broadcast(me, tagbase+me, sizeof(int), &size);
	    this->broadcast(me, tagbase+me*2, size*sizeof(int), tbuf.getData());
	  }
	else
	  {
	    int tag = tagbase + k;
	    int src = k;
	    int siz = sizeof(int);
	    int cnt;
	    this->brecv(tag, src, siz, &cnt);
	    int *buf = new int[cnt];
	    tag = tagbase + 2*k;
	    siz = cnt * sizeof(int);
	    this->brecv(tag, src, siz, buf);

	    mybuf.merge(cnt, buf);
	    delete[] buf;
	  }
    }
};

#endif
