//
// mapping.h
//
// Data/Computation Mapping onto processors
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
//
#if !defined(__MAPPING_H__)
#define __MAPPING_H__

#include <math.h>
#include "mpapi.h"
#include "vector.h"

class TorusMap : public mpapi {
  int nxdim, nydim;   

public:
  TorusMap( void )
    {
      nxdim = nydim = 0;
    }

  TorusMap( mpapi &mp )
    {
      me = mp.myProc();
      np = mp.numProcs();
      
      nydim = (int) sqrt((double)np);
      nxdim = np / nydim;
    }

  int map( int x, int y )
    {
      return ((x * nydim) + nydim) % np; 
    }

  int rowSize( void )
    {
      return nxdim;
    }

  int colSize( void )
    {
      return nydim;
    }

  int isRowMember( int row )
    {
      for (int k = 0; k < nxdim; k++)
	if (this->map(k, row) == me) return 1;

      return 0;
    }

  Vector procRow( int row )
    {
      Vector pv(nxdim, -1);
      for (int k = 0; k < nxdim; k++)
	pv.setElem(k, this->map(k, row));
      return pv;
    }

  Vector procCol( int col )
    {
      Vector pv(nydim, -1);
      for (int k = 0; k < nydim; k++)
	pv.setElem(k, this->map(col, k));
      return pv;
    }
};

#endif
