//
// mpapi.h
//
// Common Message Passing API class
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
//
# if !defined(__MPAPI_H__)
#define __MPAPI_H__

#include <iostream.h>

extern "C" double w_seconds (void);

#define block_size 100

class mpapi {
protected:
  int me, np;

public:
  mpapi( void )
    {
      me = np = 0;
    }

  mpapi( int mid, int nproc )
    {
      me = mid;
      np = nproc;
    }

  int numProcs( void )
    {
      return np;
    }

  int myProc( void )
    {
      return me;
    }

  int nextProc( void )
    {
      return (me + 1) % np;
    }

  int prevProc( void )
    {
      return (me > 0) ? (me - 1) : (np - 1);
    }

  double clock( void )
    {
      return w_seconds();
    }

  // mapping of a column to a processor in symbolic fact
  int colMap(int col)
  {
    return (col/block_size) % np;
  }

  int colMapStride( void )
  {
    return block_size;
  }

  // maps index to a processor using cyclic mapping
  int cyclicMap(int index)
    {
      return (index % np);
    }

  // stride between two indices of a processor under cyclic mapping
  int cyclicStride( void )
    {
      return np;
    }

  // base index for a cyclic mapping
  int cyclicBase( void )
    {
      return (me == 0) ? np : me;
    }

  void print( void )
    {
      cout << "\t np = " << np << " me = " << me << "\n";
    }

  void print( char *msg )
    {
      cout << msg << "\n";
    }

  void printOnZero( char *msg )
    {
      if (me != 0) return;
      cout << msg << "\n";
      cout.flush();
    }

  void printOnZero( char *msg, char *msg2 )
    {
      if (me != 0) return;
      cout << msg << msg2 << "\n";
      cout.flush();
    }
};

#endif
