//
// sp.h
//
// Implementation of Message Passing API for IBM SP Systems.
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
//
# if !defined(__SP_H__)
#define __SP_H__

#include <mpproto.h>
#include "mpapi.h"


class ibmsp : public mpapi {
public:
  ibmsp( void )
    {
      mpc_environ(&np, &me);
    };

  void init(void)
    {
      mpc_environ(&np, &me);
    };

  int asend(int to, int tag, int size, void *buf)
    {
      int tmp;
      mpc_send(buf, size, to, tag, &tmp);
      return tmp;
    }

  int arecv(int &tag, int &src, int size, void *buf)
    {
      int mid;
      mpc_recv(buf, size, &src, &tag, &mid);
      return mid;
    }

  int mwait(int id)
    {
      size_t size;
      mpc_wait(&id, &size);
      return (int) size;
    }

  void bsend(int to, int tag, int size, void *buf)
    {
      mpc_bsend(buf, size, to, tag);
    }

  void brecv(int &tag, int &src, int &size, void *buf)
    {
      size_t sv = size;
      mpc_brecv(buf, size, &src, &tag, &sv);
      size = sv;
    }

  void broadcast(int root, int tag, int size, void *buf)
    {
      if (me == root)
	{
	  for (int k = 0; k < np; k++)
	    if (k != me)
	      this->bsend(k, tag, size, buf);
	}
      else
	{
	  this->brecv(tag, root, size, buf);
	}
    }
};

#endif
