//
// vector.h
//
// Vector operations library
//
// Author : Mustafa Uysal <uysal@cs.umd.edu>
//
#if !defined(__VECTOR_H__)
#define __VECTOR_H__

#include <iostream.h>

class Vector {
  int  size;
  int *data;

public:
  Vector( void )
  {
    size = 0;
    data = NULL;
  };

  Vector( int cnt, int val = 0)
  {
    size = cnt;
    data = new int[cnt];
    for (int i = 0; i < cnt; i++)
      data[i] = val;
  };

  ~Vector( void )
  {
    if (data) delete data;
  };

  int checkIndex(int ndx)
  {
    if ((ndx<0)||(ndx>size))
      {
	cerr << "\tError : index out of range, ndx = " << ndx << ".\n";
	return 1;
      }
    
    return 0;
  };

  int getSize(void)
  {
    return size;
  };

  void setElem(int ndx, int elem)
  {
    if (checkIndex(ndx)) return;
    data[ndx] = elem;
  };

  int getElem(int ndx)
  {
    if (checkIndex(ndx)) return 0;
    else return data[ndx];
  };

  void expand(int extra)
  {
    if (extra <= 0) return;

    int *nbuf = new int[extra+size];
    for (int k = 0; k < size; k++)
      nbuf[k] = data[k];

    if (data) delete data;

    data  = nbuf;
    size += extra;
  };


  void merge(int cnt, int valid, int *ndata)
  {
    if (cnt <= 0) return;
    int *nbuf = new int[cnt+size];
    int  i = 0, j = 0;

    for (int k = 0; (i < valid) && (j < cnt); k++)
      if (data[i] < ndata[j]) nbuf[k] = data[i++];
      else nbuf[k] = ndata[j++];
    while (i < valid)
      nbuf[k++] = data[i++];
    while (j < cnt)
      nbuf[k++] = ndata[j++];
    
    if (data) delete data;
    data  = nbuf;
    size += cnt;
  }

  void sort( int cnt )
  {
    for (int i = 0; i < cnt; i++)
      for (int j = i+1; j < cnt; j++)
	if (data[j]<data[i])
	  {
	    int tmp = data[j];
	    data[j] = data[i];
	    data[i] = tmp;
	  }
  }
};


#endif
