/*
 * world.h
 *
 * $Source: /fs/nene/uysal/pf/Part/RCS/world.h,v $
 *
 * $Header: /fs/nene/uysal/pf/Part/RCS/world.h,v 1.3 1995/10/09 19:36:08 uysal Exp uysal $
 *
 * This file contains definitions of an environment that each source file 
 * should have. It basically defines the "world" that a source file lives in.
 *
 * Author : Mustafa Uysal   <uysal@cs.umd.edu>
 *
 */
#ifndef __WORLD_H__
#define __WORLD_H__

#include <stdio.h>
#include <assert.h>


/*
 * Miscellaneous Constants
 */
#define OK             0           
#define ERROR         -1         



/* 
 * Boolean Constants
 */
#define TRUE           1
#define FALSE          0



/*
 * Switch Values
 */
#define  ON            1
#define  OFF           0



/*
 * Definitions for Argument Declarations
 */
#define  ARGS(x)         ()
#define  IN
#define  IN_OUT
#define  OUT



/*
 * Definitions for Variable Declarations
 *
 */
#define  EXPORT
#define  IMPORT      extern
#define  PRIVATE     
#define  PUBLIC



/*
 * Common Arithmetic Macros 
 */
#if !defined(MIN)
#define MIN(x,y)     (((x) < (y)) ? (x) : (y))
#endif

#if !defined(MAX)
#define MAX(x,y)     (((x) < (y)) ? (y) : (x))
#endif



/*
 * Macros for code development 
 *
 */
#define MSG_LEN          200
#define ASSERT(exp)      assert(exp)
#define MESSAGE(arg)     { char msg[MSG_LEN]; sprintf arg;\
			     printf("\n%s", msg); fflush(stdout); }
#define ERROR_MSG(arg)   { char msg[MSG_LEN]; sprintf arg;\
			     printf("\n(Error) %s", msg); }
#define WARNING_MSG(arg) { char msg[MSG_LEN]; sprintf arg;\
			     printf("\n(Warning) %s", msg); }
#define DBG_MSG(exp,act) { char msg[MSG_LEN];\
			     if (exp)\
			     {  sprintf act;\
				printf("\n %s", msg);\
			      }\
			 }
#define SANITY_CHK(exp,act) { char msg[MSG_LEN];\
			     if (exp)\
			     {  sprintf act;\
				printf("\n** %s **\n", msg);\
				exit(1);\
			      }\
			 }
#define PANIC(arg)       { char msg[MSG_LEN]; sprintf arg;\
			     printf("\n*Error* %s[%d] - %s\n\n", __FILE__, __LINE__, msg);\
			     fflush(stdout); exit(1); }

#define EXIT_IF(exp,act) { char msg[MSG_LEN];\
			     if (exp)\
			     {  sprintf act;\
				printf("\n*Error in %s[%d] - %s\n\n", __FILE__, __LINE__, msg);\
				fflush(stdout);  exit(1);\
			      }\
			 }


#define STUB(x)          { printf("\nSTUB Function : %s", x); }
			     
#define CHKNULL(ptr)   \
  if (ptr == NULL) {   \
    printf("\nNull pointer encounter at file %s line %d.\n", __FILE__, __LINE__); \
    exit(1); \
  }
#define CHKFILE(ptr,msg)   \
  if (ptr == NULL) {   \
    printf("\nCannot open file %s (Line %d of %s).\n", msg, __LINE__, __FILE__); \
    exit(1); \
  }
#define DBG_ACT(exp,act) { if (exp) { act; } }


#define MALCHK(ptr,siz)\
   EXIT_IF(!ptr, (msg, "malloc(%d) failed", siz))

#define CALCHK(ptr,cnt,siz)\
   EXIT_IF(!ptr, (msg, "calloc(%d,%d) failed", cnt, siz))

#define NEWLINE      putchar('\n')
#define POSITIVE(x)  (((x) > 0) ? x : 0);

#define MEGABYTE  0x100000
#define TOOMANY   (2*MEGABYTE)


double CLOCK();


#endif __WORLD_H__




