/*
 * main.c
 *
 * $Source: /fs/nene/uysal/pf/SymFact/RCS/main.c,v $
 *
 * $Header: /fs/nene/uysal/pf/SymFact/RCS/main.c,v 1.3 1995/09/28 04:19:38 uysal Exp $
 *
 * Parallel Out-of-core Symbolic Factorization 
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>
#include <ctype.h>

#include "world.h"
#include "ipc.h"


/*--------------- Function Prototypes ---------------*/

void usage();
void GetParameters();
void SymbolicFactor();
void ReadColumnIndices();

/*--------------- Global Variables ---------------*/

char *progname;    /* Binary executable of program */
int  verbose;      /* Controls the progress reports to the user */
int  nz_size;      /* Machine dependent size of a single nonzero in bytes */

/*--------------- Implementation ---------------*/

main(argc,argv)
     int argc;
     char **argv;
{
  char *infile, *outfile;   /* file names */
  int  ncol, lcol;          /* global & local column counts */
  int  *sndNdx, sndCnt;     /* Supernode structure of the matrix */

  double sf_time;   /* variables for timing */
  double dr_time;
  double t0,t1;
  double CLOCK();
  
  dr_time = sf_time = 0.0;
  
  /* Initialization */
  MPPinit();
  verbose = 1;     /* set verbose ON */

  /* Parameter processing */
  GetParameters(argc, argv, &infile, &outfile, &nz_size);

  /* Informative messages */
  if (!MyProc())
    {
      fprintf(stderr, "\nParallel Symbolic Factorization\n");
      fprintf(stderr, "\n\tInput : %s", infile);
      fprintf(stderr, "\n\tOutput : %s", outfile);
      fprintf(stderr, "\n\tNonzero size is %d bytes", nz_size);
      fprintf(stderr, "\n");
    }

  
  /* Read in the sparse matrix structure */
  t0 = CLOCK();
  ReadColIndices(infile, &ncol, &lcol);
  t1 = CLOCK();
  dr_time = (t1 - t0);

  /* Symbolic Factorization */
  t0 = CLOCK();
  SymbolicFactor(ncol, lcol, outfile);
  t1 = CLOCK();
  sf_time = (t1 - t0);

  printf("\nInputRead:[%.3lf secs], SymFact:[%.3lf secs]\n", dr_time, sf_time);
  
  if (!MyProc()) printf("\n\n");
  exit(0);
}





/*
 * GetParameters()
 *
 * Process the command line parameters and set the working mode
 * variables for the main processing unit 
 *
 */
void GetParameters(argc, argv, infptr, outfptr, nzptr)
    int argc;
    char *argv[];
    char **infptr, **outfptr;
    int *nzptr;
{
  int  argndx, i;

  /* get the executables' name */
  progname = argv[0];

  /* input control police */
  if (argc<4) usage();

  /* input file */
  argndx = 1;
  *infptr = argv[argndx++];
  *outfptr = argv[argndx++];

  if (tolower(argv[argndx][0]) == 'f')
    *nzptr = sizeof(float);
  else if (tolower(argv[argndx][0]) == 'd')
    *nzptr = sizeof(double);
  else if (tolower(argv[argndx][0]) == 'z' && tolower(argv[argndx][1]) == 'd')
    *nzptr = sizeof(double) * 2;
  else if (tolower(argv[argndx][0]) == 'z' && strlen(argv[argndx]) == 1)
    *nzptr = sizeof(float) * 2;
  else
    PANIC((msg, "Error : unable to process argument : %s", argv[argndx]));
}







/*
 * usage()
 *
 * Early exit from the program with a display of program usage
 *
 */
void usage()
{
  fprintf(stderr, "\n\n");
  fprintf(stderr, "\nUsage : %s inputFile outputFile [F, D, Z, ZD]\n", progname);
  fprintf(stderr, "\n\n");
  exit(1);
}







