/*
 * queue.c
 *
 * $Source: /fs/nene/uysal/mambo/cholesky/symbolic/RCS/queue.c,v $
 *
 * $Header: /fs/nene/uysal/mambo/cholesky/symbolic/RCS/queue.c,v 1.1 1995/09/08 05:53:32 uysal Exp $
 *
 * a queue implementation. 
 * 
 * Author    : Anurag Acharya <acha@cs.umd.edu>
 * Revisions : Mustafa Uysal  <uysal@cs.umd.edu>
 *
 */
#include <assert.h>

#include "world.h"
#include "queue.h"

#define INIT_QUEUE_SIZE    32


/* routine to make a queue */
queue_t *QueueNew()
{
  queue_t *q = (queue_t *) malloc(sizeof(queue_t));

  q->base = (caddr_t *) malloc(sizeof(caddr_t)*INIT_QUEUE_SIZE);

  q->head = -1;
  q->tail = 0;
  q->size = INIT_QUEUE_SIZE;
  q->load = 0;
  
  return q;
}


/* routine to extend a queue */
queue_t *queue_extend(q)
    queue_t *q;
{
  caddr_t *newq;
  int i, j;

  newq = (caddr_t *) malloc(sizeof(caddr_t)*q->size*2);
  CHKNULL(newq);

  /* copy the old queue into the new space and pack it in straight */
  for (i = q->head, j = 0; j < q->size; j++)
  {
    newq[j] = q->base[i];
    if (++i == q->size)
      i = 0;
  }
  
  free(q->base);

  q->base = newq;
  q->head = 0;
  q->tail = q->size;
  q->size *= 2;
  
  return(q); 
}




/* routine to free a queue */
void QueueDestroy(queue_t *q)
{
  free(q->base);
  free(q);
}



/* routine to add an item to the queue */
void QueueAdd(q,item)
    queue_t *q;
    caddr_t item;
{
  assert(item);  /* I don't accept NULL ptrs */

  q->load++;
  q->base[q->tail] = item;
  
  if (q->head == -1)
    q->head = 0;

  q->tail = (q->tail+1) % q->size;
  if (q->tail == q->head)
    queue_extend(q);
}



/* routine to delete an item from the queue */
caddr_t QueueDelete(q)
    queue_t *q;
{
  int ndx;
  
  if (q->head == -1)
    {
      assert(q->load == 0);
      *((int*)(q->load)) = 0;  /* get a SEGV signal */
      return NULL;
    }

  q->load--;
  ndx = q->head;
  q->head = (q->head+1) % q->size;
  
  if (q->head == q->tail)
    {
      q->head = -1;
      q->tail = 0;
      q->load = 0;
    }

  assert(q->base[ndx]);
  return(q->base[ndx]);
}



/* routine to probe the head of the queue */
caddr_t QueueHead(q)
    queue_t *q;
{
  if (q->head == -1)
    return (caddr_t) NULL;
  else
    return q->base[q->head];
}



/* routine to see if a queue is empty or not */
int QueueEmpty(q)
    queue_t *q;
{
  if (q->head == -1)
    {
      assert(q->load == 0);
      return 1;
    }

  return 0;
}

