/*
 * symfact.c
 *
 * $Source: /fs/nene/uysal/pf/SymFact/RCS/symfact.c,v $
 *
 * $Header: /fs/nene/uysal/pf/SymFact/RCS/symfact.c,v 1.4 1995/09/28 04:19:38 uysal Exp $
 *
 * Module to implement parallel out-of-core symbolic factorization
 *
 * Author : Mustafa Uysal <uysal@cs.umd.edu>
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>

#include "world.h"
#include "mppworld.h"
#include "ipc.h"

#define FULL_SEND  1
#define SIG_SEND   0
#define MAX_ASYNC_MSG  1000

/*--------------- Static Global Variables ---------------*/
int  **Matrix;        /* Storage for Matrix Nonzero Structure */
static int  colcnt;   /* Column counter */
static int *recva, *recvb;
int block_size = 100;
static int msgcnt = 0;
static int me,np;
static int gnc,glc;

/*--------------- Function Prototypes ---------------*/
int *ColumnReceive();
int *IndexSetMerge();
void SendOutColumn();
int  IndexSetCompare();
int  ColNdx();


/*--------------- Implementation ---------------*/


/*
 * Parallel Symbolic Matrix Factorization for very large matrices
 * that uses minimal amount of memory, yet fast enough so that
 * something gets done in reasonable time
 */
void SymbolicFactor(nc, lc, outfile)
    int nc;      /* number of columns in the whole matrix */
    int lc;      /* number of columns that this processor has */
    char *outfile;
{
  int outf;      /* binary file to hold the local matrix structure */
  int lndx;      /* Local index of a local column */
  int rincol;    /* No. of rows in the column being processed */
  int *colptr;   /* Pointer to the row indices of the current column */
  int to;        /* Index into current column rows denoting the target column */
  int tocol;     /* Column number of target column */
  int dndx;      /* local index of destination column */
  int *dst;      /* Pointer to the destination column structure */
  int *newloc;   /* Pointer to fresh memory */
  int i,j;       /* loop variables */
  int cnt, prevcmp;
  int col, slab, sta;
  int dsiz, nullcol;
  double sf_wrt_bytes, sf_wrt_rate;

  double t0,t1,sf_comm_time;
  double sf_ndx_merge, sf_ndx_compare, sf_wrt_time;
  
  gnc = nc;
  glc = lc;

  sf_comm_time = sf_ndx_merge = sf_ndx_compare = sf_wrt_time = 0.0;
  
  /* initialization */
  me = MyProc();
  np = NumProcs();
  sf_wrt_bytes = sf_wrt_rate = 0.0;
  colcnt = 1;
  recva = (int*) calloc(nc, sizeof(int));
  CHKNULL(recva);
  newloc = recvb = (int*) calloc(nc, sizeof(int));
  CHKNULL(recvb);
  
  /* open local file and write header recs */
  outf = open(outfile, O_CREAT | O_TRUNC | O_WRONLY, 0644);
  EXIT_IF(outf<0, (msg, "Error in opening output file [%s]", outfile));

  TimedWrite(outf, &nc, sizeof(int), &sf_wrt_time, &sf_wrt_bytes);
  TimedWrite(outf, &np, sizeof(int), &sf_wrt_time, &sf_wrt_bytes);
  TimedWrite(outf, &me, sizeof(int), &sf_wrt_time, &sf_wrt_bytes);

  /* main process loop */
  for (col=slab=1; col<=nc; slab++)
    if (ColMap(col) == me)
      {
	for ( sta=col; col <= MIN(nc, slab*block_size-1); col++)
	  {
	    t0 = CLOCK();

	    lndx   = ColNdx(col);
	    colptr = Matrix[lndx];
	    rincol = colptr[0];

	    printf("atCol:%d [%d], %d nonzeros\n", col, colptr[1], rincol);

	    for (to=2; to<rincol; to++)
	      {
		tocol = colptr[to];
		if (ColMap(tocol) == me)
		  {
		    if (tocol<=col || tocol>nc)
		      PrintColumn(col, colptr);

		    EXIT_IF(tocol>nc||tocol<=col, 
			    (msg, "Invalid row index(%d@%d) found in column(%d) of size(%d-%d)",
			     tocol, to, col, colptr[0], rincol));

		    dndx = ColNdx(tocol);
		    dst  = Matrix[dndx];
		    EXIT_IF(dst[1]!=tocol,
			    (msg, "Update Error : tc:%d d0:%d d1:%d me:%d map:%d @col:%d",
			     tocol, dst[0], dst[1], me, ColMap(tocol), col));

		    Matrix[dndx] = IndexSetMerge(colptr, to, dst);
		    /*IndMergeChk(Matrix[dndx], colptr, to);*/
		  }
	      }
	    t1 = CLOCK();
	    sf_ndx_merge += (t1-t0);

	    if (msgcnt>MAX_ASYNC_MSG)
	      {
		SendQueueDrain(col);
		msgcnt = 0;
	      }

	    if (col>sta)
	      {
		if (IndexSetCompare(Matrix[ColNdx(col-1)], Matrix[ColNdx(col)]) == TRUE)
		  SendOutColumn(Matrix[ColNdx(col-1)], col-1, SIG_SEND);
		else
		  SendOutColumn(Matrix[ColNdx(col-1)], col-1, FULL_SEND);
		msgcnt += np;
	      }	    
	    t0 = CLOCK();
	    sf_comm_time += (t0 - t1);

	    t0 = CLOCK();
	    TimedWrite(outf, colptr, rincol*sizeof(int), &sf_wrt_time, &sf_wrt_bytes);
	    t1 = CLOCK();
	    sf_wrt_time += (t1 - t0);
	  }

	SendOutColumn(Matrix[ColNdx(col-1)], col-1, FULL_SEND);

	/* Make sure no active column is left */
	SendQueueDrain(col);
	
	/* dispose all the columns in the current chunk */
	for ( col=sta; col <= MIN(nc, slab*block_size-1); col++)	
	  {
	    lndx   = ColNdx(col);
	    colptr = Matrix[lndx];
	    free(colptr);
	  }
      }
    else
      {
	for (nullcol=0; col <= MIN(nc, slab*block_size-1); col++)
	  {
	    t0 = CLOCK();
	    colptr = ColumnReceive(colptr, col, recva, recvb, nc);
	    t1 = CLOCK();
	    sf_comm_time += (t1-t0);

	    if (colptr == NULL)
	      {
		newloc[++nullcol] = col;
		continue;
	      }

	    t0 = CLOCK();
	    rincol = colptr[0] + nullcol;
	    if (nullcol)
	      {
		newloc[0] = rincol;
		for (i=nullcol+1,j=1; j<colptr[0]; i++, j++)
		  newloc[i] = colptr[j];
		/*for (i=0; i<=nullcol; i++)
		  printf("\nNULL-%d-COL[%d] = %d", nullcol, i, newloc[i]);*/
		nullcol = 0;
		colptr = newloc;
	      }
	    for (to=2; to<rincol; to++)
	      {
		tocol = colptr[to];
		if (ColMap(tocol) == me)
		  {
		    dndx = ColNdx(tocol);
		    dst  = Matrix[dndx];
		    EXIT_IF(dst[1]!=tocol,
			    (msg, "Update Error : tc:%d [%d %d] map:%d ndx:%d",
			     tocol, dst[0], dst[1], ColMap(tocol), ColNdx(tocol)));
		    /*printf("\n*(c:%d[%d], t:%d[%d], ct:%d d:%d) -> [%d]",
			   col, colptr[0], tocol, dst[0], to, dndx, Matrix[150][1]);*/
		    Matrix[dndx] = IndexSetMerge(colptr, to, dst);
		  }
	      }
	    t1 = CLOCK();
	    sf_ndx_merge += (t1-t0);
	  }
      }


  nc = 1;
  assert(write(outf, &nc, sizeof(int))==sizeof(int));
  nc = -1;
  assert(write(outf, &nc, sizeof(int))==sizeof(int));
  close(outf);

  printf("\nComm:[%.3lf secs], Merge:[%.3lf secs] Write:[%.0lf bytes, %.3lf secs, %.3lf MB/sec]",
	 sf_comm_time, sf_ndx_merge, sf_wrt_bytes, sf_wrt_time,
	 sf_wrt_bytes/(sf_wrt_time*MEGABYTE));
  fflush(stdout);
  Sync();
  
  /* postprocessing */
  free(Matrix);
  free(recva);
  free(recvb);
}     





/*
 * Function to merge two index sets. The second argument is freed
 * inside the function and the merged index set is returned to
 * the caller.
 */
int *IndexSetMerge(set1, cnt, set2)
    int *set1;
    int *set2;
    int cnt;
{
  register int  i,j,k;
  int *scatter;
  int  nset1, nset2;

  /* Index Set Merge */
  nset1 = set1[0];
  nset2 = set2[0];

  EXIT_IF(set1[cnt]!=set2[1], (msg, "Index Merge Error [%d %d %d %d %d %d]",
			       cnt, set1[cnt], set2[1], set1[1], nset1, nset2));

  /* memory allocation */
  scatter = (int*) calloc(nset1+nset2-cnt, sizeof(int));
  if (!scatter) exit(0);

  for (i=cnt, j=k=1; (i<nset1) && (j<nset2); k++)
    {
      if (set1[i] < set2[j])
	scatter[k] = set1[i++];
      else if (set1[i] > set2[j])
	scatter[k] = set2[j++];
      else
	{
	  scatter[k] = set1[i];
	  j++; i++;
	}
    }
  for ( ; i<nset1; )
    scatter[k++] = set1[i++];
  for ( ; j<nset2; )
    scatter[k++] = set2[j++];

  /* free up the second argument */
  free(set2);
  
  /* copy of new index set into new location */
  scatter[0] = k;
  
  return scatter;
}








/*
 * Function to compare two index sets. It is used to
 * determine whether two columns belong to the same supernode
 * or not.
 */
int IndexSetCompare(set1, set2)
    int *set1;
    int *set2;
{
  int  nset1, nset2;
  int  i,j,k;

  /* one of the index sets are NULL */
  if (!set1 || !set2)
    return FALSE;
  
  /* Get the index vector sizes */
  nset1 = set1[0];
  nset2 = set2[0];

  /* not identical based on size info */
  if ((nset1 - 1) != nset2)
    return FALSE;

  /* need to compare sets to determine the structural identity */
  for (i=2, j=1; i<nset1; i++, j++)
    if (set1[i] != set2[j])
      return FALSE;

  return TRUE;
}





int *ColumnReceive(data, col, bufa, bufb, cnt)
    int *data;
    int col;
    int *bufa, *bufb;
    int cnt;
{
  int i, to;

  MsgRecv(MSG__DATA+col, bufa, cnt*sizeof(int));
  if (bufa[0] == -1)
    return NULL;
  else
    return bufa;
}





void SendOutColumn(data, col, mode)
    int *data, col, mode;
{
  static int amsg = -1;
  int i, to;

  if ((col<0) || (col > gnc))
    {
      printf("Error: invalid column:%d (max:%d)\n", col, gnc);
      assert(0);
    }

  if (mode == FULL_SEND)
    {
      for (i=1; i<np; i++)
	{
	  to = (me + i) % np;
	  CommAsyncSend(data, to, MSG__DATA+col, data[0]*sizeof(int));
	}
    }
  else
    {
      for (i=1; i<np; i++)
	{
	  to = (me + i) % np;
	  CommAsyncSend(&amsg, to, MSG__DATA+col, sizeof(int));
	}
    }
  
  DBG_MSG(!(col%1000), (msg, "@col:%d", data[1]));
}





int ColMap(col)
    int col;
{
  return ((col/block_size) % NumProcs());
}




int ColNdx(col)
    int col;
{
  return (col/(block_size*NumProcs()))*block_size + col % block_size;
}




int PrintColumn(id, data)
    int id, *data;
{
  int i;
  
  printf("\n\tColumn:%d Size:%d Id:%d", data[1], data[0], id);
  for (i=2; i<data[0]; i++)
    printf("\n\t%d> %d", i, data[i]);
}




IndMergeChk(nc, oc, to)
    int *nc, *oc, to;
{
  int i;

  for (i=2; i<nc[0]; i++)
    if (nc[i]<=nc[i-1])
      {
	PrintColumn(-1, oc);
	PrintColumn(-2, nc);
	EXIT_IF(nc[i]<=nc[i-1], (msg, "Trouble @col:%d after:%d nd:%d", nc[1], oc[1], to));
      }
}
